@@ -1,13 +1,3 @@
-#include <sys/types.h>
-#include <unistd.h>
-#include <errno.h>
-#include <fcntl.h>
-#include <string.h>
-#include <stdlib.h>
-#include <ctype.h>
-#include <assert.h>
-#include <signal.h>
-
 #include "buffer.h"
 #include "server.h"
 #include "keyvalue.h"
@@ -23,6 +13,16 @@
 
 #include "inet_ntop_cache.h"
 
+#include <sys/types.h>
+#include <unistd.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <string.h>
+#include <stdlib.h>
+#include <ctype.h>
+#include <assert.h>
+#include <signal.h>
+
 #include <stdio.h>
 
 #ifdef HAVE_SYS_FILIO_H
@@ -32,12 +32,15 @@
 #include "sys-socket.h"
 
 #ifdef HAVE_SYS_UIO_H
-#include <sys/uio.h>
+# include <sys/uio.h>
 #endif
+
 #ifdef HAVE_SYS_WAIT_H
-#include <sys/wait.h>
+# include <sys/wait.h>
 #endif
 
+#include "version.h"
+
 enum {EOL_UNSET, EOL_N, EOL_RN};
 
 /*
@@ -329,7 +332,20 @@ static handler_t scgi_handle_fdevent(void *s, void *ctx, int revents);
 
 int scgi_proclist_sort_down(server *srv, scgi_extension_host *host, scgi_proc *proc);
 
-
+static void reset_signals(void) {
+#ifdef SIGTTOU
+	signal(SIGTTOU, SIG_DFL);
+#endif
+#ifdef SIGTTIN
+	signal(SIGTTIN, SIG_DFL);
+#endif
+#ifdef SIGTSTP
+	signal(SIGTSTP, SIG_DFL);
+#endif
+	signal(SIGHUP, SIG_DFL);
+	signal(SIGPIPE, SIG_DFL);
+	signal(SIGUSR1, SIG_DFL);
+}
 
 static handler_ctx * handler_ctx_init() {
 	handler_ctx * hctx;
@@ -372,7 +388,7 @@ static void handler_ctx_free(handler_ctx *hctx) {
 	free(hctx);
 }
 
-scgi_proc *scgi_process_init() {
+static scgi_proc *scgi_process_init() {
 	scgi_proc *f;
 
 	f = calloc(1, sizeof(*f));
@@ -384,7 +400,7 @@ scgi_proc *scgi_process_init() {
 	return f;
 }
 
-void scgi_process_free(scgi_proc *f) {
+static void scgi_process_free(scgi_proc *f) {
 	if (!f) return;
 
 	scgi_process_free(f->next);
@@ -394,7 +410,7 @@ void scgi_process_free(scgi_proc *f) {
 	free(f);
 }
 
-scgi_extension_host *scgi_host_init() {
+static scgi_extension_host *scgi_host_init() {
 	scgi_extension_host *f;
 
 	f = calloc(1, sizeof(*f));
@@ -409,7 +425,7 @@ scgi_extension_host *scgi_host_init() {
 	return f;
 }
 
-void scgi_host_free(scgi_extension_host *h) {
+static void scgi_host_free(scgi_extension_host *h) {
 	if (!h) return;
 
 	buffer_free(h->host);
@@ -426,7 +442,7 @@ void scgi_host_free(scgi_extension_host *h) {
 
 }
 
-scgi_exts *scgi_extensions_init() {
+static scgi_exts *scgi_extensions_init() {
 	scgi_exts *f;
 
 	f = calloc(1, sizeof(*f));
@@ -434,7 +450,7 @@ scgi_exts *scgi_extensions_init() {
 	return f;
 }
 
-void scgi_extensions_free(scgi_exts *f) {
+static void scgi_extensions_free(scgi_exts *f) {
 	size_t i;
 
 	if (!f) return;
@@ -464,7 +480,7 @@ void scgi_extensions_free(scgi_exts *f) {
 	free(f);
 }
 
-int scgi_extension_insert(scgi_exts *ext, buffer *key, scgi_extension_host *fh) {
+static int scgi_extension_insert(scgi_exts *ext, buffer *key, scgi_extension_host *fh) {
 	scgi_extension *fe;
 	size_t i;
 
@@ -770,10 +786,8 @@ static int scgi_spawn_connection(server *srv,
 			env.used = 0;
 
 			if (scgi_fd != 0) {
-				close(0);
 				dup2(scgi_fd, 0);
 				close(scgi_fd);
-				scgi_fd = 0;
 			}
 
 			/* we don't need the client socket */
@@ -781,8 +795,6 @@ static int scgi_spawn_connection(server *srv,
 				close(fd);
 			}
 
-			openDevNull(STDERR_FILENO);
-
 			/* build clean environment */
 			if (host->bin_env_copy->used) {
 				for (i = 0; i < host->bin_env_copy->used; i++) {
@@ -826,6 +838,8 @@ static int scgi_spawn_connection(server *srv,
 			buffer_copy_string_len(b, CONST_STR_LEN("exec "));
 			buffer_append_string_buffer(b, host->bin_path);
 
+			reset_signals();
+
 			/* exec the cgi */
 			execle("/bin/sh", "sh", "-c", b->ptr, (char *)NULL, env.ptr);
 
@@ -1178,7 +1192,7 @@ static int scgi_set_state(server *srv, handler_ctx *hctx, scgi_connection_state_
 }
 
 
-void scgi_connection_cleanup(server *srv, handler_ctx *hctx) {
+static void scgi_connection_cleanup(server *srv, handler_ctx *hctx) {
 	plugin_data *p;
 	connection  *con;
 
@@ -1281,10 +1295,11 @@ static int scgi_env_add(buffer *env, const char *key, size_t key_len, const char
 
 	buffer_prepare_append(env, len);
 
-	/* include the NUL */
-	memcpy(env->ptr + env->used, key, key_len + 1);
+	memcpy(env->ptr + env->used, key, key_len);
+	env->ptr[env->used + key_len] = '\0';
 	env->used += key_len + 1;
-	memcpy(env->ptr + env->used, val, val_len + 1);
+	memcpy(env->ptr + env->used, val, val_len);
+	env->ptr[env->used + val_len] = '\0';
 	env->used += val_len + 1;
 
 	return 0;
@@ -1461,10 +1476,18 @@ static int scgi_create_env(server *srv, handler_ctx *hctx) {
 	scgi_env_add(p->scgi_env, CONST_STR_LEN("SCGI"), CONST_STR_LEN("1"));
 
 
-	scgi_env_add(p->scgi_env, CONST_STR_LEN("SERVER_SOFTWARE"), CONST_STR_LEN(PACKAGE_NAME"/"PACKAGE_VERSION));
+	if (buffer_is_empty(con->conf.server_tag)) {
+		scgi_env_add(p->scgi_env, CONST_STR_LEN("SERVER_SOFTWARE"), CONST_STR_LEN(PACKAGE_DESC));
+	} else {
+		scgi_env_add(p->scgi_env, CONST_STR_LEN("SERVER_SOFTWARE"), CONST_BUF_LEN(con->conf.server_tag));
+	}
 
 	if (con->server_name->used) {
-		scgi_env_add(p->scgi_env, CONST_STR_LEN("SERVER_NAME"), CONST_BUF_LEN(con->server_name));
+		size_t len = con->server_name->used - 1;
+		char *colon = strchr(con->server_name->ptr, ':');
+		if (colon) len = colon - con->server_name->ptr;
+
+		scgi_env_add(p->scgi_env, CONST_STR_LEN("SERVER_NAME"), con->server_name->ptr, len);
 	} else {
 #ifdef HAVE_IPV6
 		s = inet_ntop(srv_sock->addr.plain.sa_family,
@@ -1915,7 +1938,7 @@ static int scgi_demux_response(server *srv, handler_ctx *hctx) {
 }
 
 
-int scgi_proclist_sort_up(server *srv, scgi_extension_host *host, scgi_proc *proc) {
+static int scgi_proclist_sort_up(server *srv, scgi_extension_host *host, scgi_proc *proc) {
 	scgi_proc *p;
 
 	UNUSED(srv);
@@ -2152,8 +2175,11 @@ static handler_t scgi_write_request(server *srv, handler_ctx *hctx) {
 	int ret;
 
 	/* sanity check */
-	if (!host ||
-	    ((!host->host->used || !host->port) && !host->unixsocket->used)) {
+	if (!host) {
+		log_error_write(srv, __FILE__, __LINE__, "s", "fatal error: host = NULL");
+		return HANDLER_ERROR;
+	}
+	if (((!host->host->used || !host->port) && !host->unixsocket->used)) {
 		log_error_write(srv, __FILE__, __LINE__, "sxddd",
 				"write-req: error",
 				host,
@@ -2288,8 +2314,8 @@ static handler_t scgi_write_request(server *srv, handler_ctx *hctx) {
 
 		chunkqueue_remove_finished_chunks(hctx->wb);
 
-		if (-1 == ret) {
-			if (errno == ENOTCONN) {
+		if (ret < 0) {
+			if (errno == ENOTCONN || ret == -2) {
 				/* the connection got dropped after accept()
 				 *
 				 * this is most of the time a PHP which dies
@@ -2314,24 +2340,17 @@ static handler_t scgi_write_request(server *srv, handler_ctx *hctx) {
 				 */
 
 				log_error_write(srv, __FILE__, __LINE__, "ssosd",
-						"[REPORT ME] connection was dropped after accept(). reconnect() denied:",
+						"connection was dropped after accept(). reconnect() denied:",
 						"write-offset:", hctx->wb->bytes_out,
 						"reconnect attempts:", hctx->reconnects);
 
 				return HANDLER_ERROR;
-			}
-
-			if ((errno != EAGAIN) &&
-			    (errno != EINTR)) {
-
+			} else {
+				/* -1 == ret => error on our side */
 				log_error_write(srv, __FILE__, __LINE__, "ssd",
-						"write failed:", strerror(errno), errno);
+					"write failed:", strerror(errno), errno);
 
 				return HANDLER_ERROR;
-			} else {
-				fdevent_event_add(srv->ev, &(hctx->fde_ndx), hctx->fd, FDEVENT_OUT);
-
-				return HANDLER_WAIT_FOR_EVENT;
 			}
 		}
 
@@ -2458,12 +2477,10 @@ SUBREQUEST_FUNC(mod_scgi_handle_subrequest) {
 }
 
 static handler_t scgi_connection_close(server *srv, handler_ctx *hctx) {
-	plugin_data *p;
 	connection  *con;
 
 	if (NULL == hctx) return HANDLER_GO_ON;
 
-	p    = hctx->plugin_data;
 	con  = hctx->remote_conn;
 
 	log_error_write(srv, __FILE__, __LINE__, "ssdsd",
@@ -2713,27 +2730,29 @@ static handler_t scgi_check_extension(server *srv, connection *con, void *p_d, i
 	/* check if extension matches */
 	for (k = 0; k < p->conf.exts->used; k++) {
 		size_t ct_len;
+		scgi_extension *ext = p->conf.exts->exts[k];
 
-		extension = p->conf.exts->exts[k];
+		if (ext->key->used == 0) continue;
 
-		if (extension->key->used == 0) continue;
-
-		ct_len = extension->key->used - 1;
+		ct_len = ext->key->used - 1;
 
 		if (s_len < ct_len) continue;
 
 		/* check extension in the form "/scgi_pattern" */
-		if (*(extension->key->ptr) == '/') {
-			if (strncmp(fn->ptr, extension->key->ptr, ct_len) == 0)
+		if (*(ext->key->ptr) == '/') {
+			if (strncmp(fn->ptr, ext->key->ptr, ct_len) == 0) {
+				extension = ext;
 				break;
-		} else if (0 == strncmp(fn->ptr + s_len - ct_len, extension->key->ptr, ct_len)) {
+			}
+		} else if (0 == strncmp(fn->ptr + s_len - ct_len, ext->key->ptr, ct_len)) {
 			/* check extension in the form ".fcg" */
+			extension = ext;
 			break;
 		}
 	}
 
 	/* extension doesn't match */
-	if (k == p->conf.exts->used) {
+	if (NULL == extension) {
 		return HANDLER_GO_ON;
 	}
 
@@ -2827,7 +2846,11 @@ static handler_t scgi_check_extension(server *srv, connection *con, void *p_d, i
 			 */
 
 			/* the rewrite is only done for /prefix/? matches */
-			if (extension->key->ptr[0] == '/' &&
+			if (host->fix_root_path_name && extension->key->ptr[0] == '/' && extension->key->ptr[1] == '\0') {
+				buffer_copy_string(con->request.pathinfo, con->uri.path->ptr);
+				con->uri.path->used = 1;
+				con->uri.path->ptr[con->uri.path->used - 1] = '\0';
+			} else if (extension->key->ptr[0] == '/' &&
 			    con->uri.path->used > extension->key->used &&
 			    NULL != (pathinfo = strchr(con->uri.path->ptr + extension->key->used - 1, '/'))) {
 				/* rewrite uri.path and pathinfo */
@@ -2836,10 +2859,6 @@ static handler_t scgi_check_extension(server *srv, connection *con, void *p_d, i
 
 				con->uri.path->used -= con->request.pathinfo->used - 1;
 				con->uri.path->ptr[con->uri.path->used - 1] = '\0';
-			} else if (host->fix_root_path_name && extension->key->ptr[0] == '/' && extension->key->ptr[1] == '\0') {
-				buffer_copy_string(con->request.pathinfo, con->uri.path->ptr);
-				con->uri.path->used = 1;
-				con->uri.path->ptr[con->uri.path->used - 1] = '\0';
 			}
 		}
 	} else {
@@ -3105,6 +3124,7 @@ TRIGGER_FUNC(mod_scgi_handle_trigger) {
 }
 
 
+int mod_scgi_plugin_init(plugin *p);
 int mod_scgi_plugin_init(plugin *p) {
 	p->version     = LIGHTTPD_VERSION_ID;
 	p->name         = buffer_init_string("scgi");
