@@ -1,16 +1,12 @@
-#include <ctype.h>
-#include <stdlib.h>
-#include <string.h>
-
 #include "base.h"
 #include "log.h"
 #include "buffer.h"
 
 #include "plugin.h"
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
+#include <ctype.h>
+#include <stdlib.h>
+#include <string.h>
 
 /**
  * this is a skeleton for a lighttpd plugin
@@ -169,6 +165,8 @@ URIHANDLER_FUNC(mod_skeleton_uri_handler) {
 
 	UNUSED(srv);
 
+	if (con->mode != DIRECT) return HANDLER_GO_ON;
+
 	if (con->uri.path->used == 0) return HANDLER_GO_ON;
 
 	mod_skeleton_patch_connection(srv, con, p);
